require "em-http-request"

module Emvoicer
  module Services
    class Base
      def request(url, params, callback)
        http = EM::HttpRequest.new(url).get(:query => params, :head => request_headers)
        http.callback { |http| callback.call(http) }
        http.errback { raise Emvoicer::RequestError }
      end

      def invoices(params, callback)
        request(invoices_url, params, proc { |http|
          invoices = handle_invoices_source(http.response)
          callback.call(invoices)
        })
      end

      def handle_invoices_source(source)
        nodes = self.class.extract_invoice_nodes(source)
        nodes.map { |n| generate_invoice(n) }
      end

      def generate_invoice(node)
        attributes = self.class.parse_invoice(node)
        Emvoicer::Invoice.new(self, attributes)
      end

      def client(id, callback)
        request(client_url(id), {}, proc { |http|
          client = handle_client_source(http.response)
          callback.call(client)
        })
      end

      def handle_client_source(source)
        node = self.class.extract_client_node(source)
        generate_client(node)
      end

      def generate_client(node)
        attributes = self.class.parse_client(node)
        Emvoicer::Client.new(self, attributes)
      end

      def request_headers
        headers
      end
      
      # Override methods below to comform speific API
      def headers
        {}
      end

      def invoices_url
        ""
      end

      def client_url(id)
        ""
      end

      def self.extract_invoice_nodes(source)
        []
      end

      def self.parse_invoice(node)
        {}
      end

      def self.extract_client_node(source)
        []
      end

      def self.parse_client(node)
        {}
      end
    end
  end

  class RequestError < StandardError
  end
end
